@@ -112,6 +112,7 @@ void DesktopGridEffect::reconfigure(ReconfigureFlags)
         effects->reserveElectricBorder(ElectricBorder(i));
     }
 
+    // TODO: rename zoomDuration to duration
     zoomDuration = animationTime(DesktopGridConfig::zoomDuration() != 0 ? DesktopGridConfig::zoomDuration() : 300);
     timeline.setCurveShape(QTimeLine::EaseInOutCurve);
     timeline.setDuration(zoomDuration);
@@ -1071,7 +1072,7 @@ void DesktopGridEffect::setup()
     qDeleteAll(hoverTimeline);
     hoverTimeline.clear();
     for (int i = 0; i < effects->numberOfDesktops(); i++) {
-        QTimeLine *newTimeline = new QTimeLine(animationTime(zoomDuration), this);
+        QTimeLine *newTimeline = new QTimeLine(zoomDuration, this);
         newTimeline->setCurveShape(QTimeLine::EaseInOutCurve);
         hoverTimeline.append(newTimeline);
     }
@@ -1292,7 +1293,7 @@ void DesktopGridEffect::desktopsAdded(int old)
     const int desktop = effects->numberOfDesktops();
     for (int i = old; i <= effects->numberOfDesktops(); i++) {
         // add a timeline for the new desktop
-        QTimeLine *newTimeline = new QTimeLine(animationTime(zoomDuration), this);
+        QTimeLine *newTimeline = new QTimeLine(zoomDuration, this);
         newTimeline->setCurveShape(QTimeLine::EaseInOutCurve);
         hoverTimeline.append(newTimeline);
     }
